"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2023.10.24
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from agilicus_api.api_client import ApiClient, Endpoint as _Endpoint
from agilicus_api.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from agilicus_api.model.bulk_delete_labelled_objects_request import BulkDeleteLabelledObjectsRequest
from agilicus_api.model.bulk_delete_labelled_objects_response import BulkDeleteLabelledObjectsResponse
from agilicus_api.model.error_message import ErrorMessage
from agilicus_api.model.label import Label
from agilicus_api.model.label_association import LabelAssociation
from agilicus_api.model.label_name import LabelName
from agilicus_api.model.labelled_object import LabelledObject
from agilicus_api.model.list_labelled_objects_response import ListLabelledObjectsResponse
from agilicus_api.model.list_labels_response import ListLabelsResponse
from agilicus_api.model.object_type import ObjectType


class LabelsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __bulk_delete_labelled_objects(
            self,
            bulk_delete_labelled_objects_request,
            **kwargs
        ):
            """Delete many label associations  # noqa: E501

            Deletes many label associations. Will also remove any labels if they are no longer referenced and they were automatically created.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.bulk_delete_labelled_objects(bulk_delete_labelled_objects_request, async_req=True)
            >>> result = thread.get()

            Args:
                bulk_delete_labelled_objects_request (BulkDeleteLabelledObjectsRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                BulkDeleteLabelledObjectsResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['bulk_delete_labelled_objects_request'] = \
                bulk_delete_labelled_objects_request
            return self.call_with_http_info(**kwargs)

        if self.bulk_delete_labelled_objects is None:
            self.bulk_delete_labelled_objects = _Endpoint(
                settings={
                    'response_type': (BulkDeleteLabelledObjectsResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects/bulk_delete',
                    'operation_id': 'bulk_delete_labelled_objects',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'bulk_delete_labelled_objects_request',
                    ],
                    'required': [
                        'bulk_delete_labelled_objects_request',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'bulk_delete_labelled_objects_request':
                            (BulkDeleteLabelledObjectsRequest,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'bulk_delete_labelled_objects_request': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__bulk_delete_labelled_objects
            )

        def __create_labelled_object(
            self,
            labelled_object,
            **kwargs
        ):
            """Add a labelled object  # noqa: E501

            Adds a new labelled object. The association must have a unique object_id, object_type and label_name within an organsation. If it does not, then the create will return a 409 including the conflicting object.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_labelled_object(labelled_object, async_req=True)
            >>> result = thread.get()

            Args:
                labelled_object (LabelledObject):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                LabelledObject
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['labelled_object'] = \
                labelled_object
            return self.call_with_http_info(**kwargs)

        if self.create_labelled_object is None:
            self.create_labelled_object = _Endpoint(
                settings={
                    'response_type': (LabelledObject,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects',
                    'operation_id': 'create_labelled_object',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'labelled_object',
                    ],
                    'required': [
                        'labelled_object',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'labelled_object':
                            (LabelledObject,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'labelled_object': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_labelled_object
            )

        def __create_labelled_object_label(
            self,
            labelled_object_id,
            label_association,
            **kwargs
        ):
            """Add a label association  # noqa: E501

            Associates a label with an object. If already associated, this will return a 409 including the conflicting object. Note that a new label will be created if one with the provided name does not yet exist.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_labelled_object_label(labelled_object_id, label_association, async_req=True)
            >>> result = thread.get()

            Args:
                labelled_object_id (str): A labelled object ID
                label_association (LabelAssociation):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                LabelAssociation
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['labelled_object_id'] = \
                labelled_object_id
            kwargs['label_association'] = \
                label_association
            return self.call_with_http_info(**kwargs)

        if self.create_labelled_object_label is None:
            self.create_labelled_object_label = _Endpoint(
                settings={
                    'response_type': (LabelAssociation,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects/{labelled_object_id}/labels',
                    'operation_id': 'create_labelled_object_label',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'labelled_object_id',
                        'label_association',
                    ],
                    'required': [
                        'labelled_object_id',
                        'label_association',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'labelled_object_id':
                            (str,),
                        'label_association':
                            (LabelAssociation,),
                    },
                    'attribute_map': {
                        'labelled_object_id': 'labelled_object_id',
                    },
                    'location_map': {
                        'labelled_object_id': 'path',
                        'label_association': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_labelled_object_label
            )

        def __create_object_label(
            self,
            label,
            **kwargs
        ):
            """Add a label  # noqa: E501

            Adds a label. Labels must have unique names within an org. If the name is not unique, a 409 will be returned.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.create_object_label(label, async_req=True)
            >>> result = thread.get()

            Args:
                label (Label):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Label
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['label'] = \
                label
            return self.call_with_http_info(**kwargs)

        if self.create_object_label is None:
            self.create_object_label = _Endpoint(
                settings={
                    'response_type': (Label,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labels',
                    'operation_id': 'create_object_label',
                    'http_method': 'POST',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'label',
                    ],
                    'required': [
                        'label',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'label':
                            (Label,),
                    },
                    'attribute_map': {
                    },
                    'location_map': {
                        'label': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__create_object_label
            )

        def __delete_labelled_object(
            self,
            labelled_object_id,
            **kwargs
        ):
            """Delete a labelled object  # noqa: E501

            Delete a labelled object  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_labelled_object(labelled_object_id, async_req=True)
            >>> result = thread.get()

            Args:
                labelled_object_id (str): A labelled object ID

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['labelled_object_id'] = \
                labelled_object_id
            return self.call_with_http_info(**kwargs)

        if self.delete_labelled_object is None:
            self.delete_labelled_object = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects/{labelled_object_id}',
                    'operation_id': 'delete_labelled_object',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'labelled_object_id',
                        'org_id',
                    ],
                    'required': [
                        'labelled_object_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'labelled_object_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'labelled_object_id': 'labelled_object_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'labelled_object_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_labelled_object
            )

        def __delete_labelled_object_label(
            self,
            labelled_object_id,
            label_name,
            **kwargs
        ):
            """Delete a label association  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_labelled_object_label(labelled_object_id, label_name, async_req=True)
            >>> result = thread.get()

            Args:
                labelled_object_id (str): A labelled object ID
                label_name (LabelName): A label name

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['labelled_object_id'] = \
                labelled_object_id
            kwargs['label_name'] = \
                label_name
            return self.call_with_http_info(**kwargs)

        if self.delete_labelled_object_label is None:
            self.delete_labelled_object_label = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects/{labelled_object_id}/labels/{label_name}',
                    'operation_id': 'delete_labelled_object_label',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'labelled_object_id',
                        'label_name',
                        'org_id',
                    ],
                    'required': [
                        'labelled_object_id',
                        'label_name',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'labelled_object_id':
                            (str,),
                        'label_name':
                            (LabelName,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'labelled_object_id': 'labelled_object_id',
                        'label_name': 'label_name',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'labelled_object_id': 'path',
                        'label_name': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_labelled_object_label
            )

        def __delete_object_label(
            self,
            label_id,
            **kwargs
        ):
            """Delete a label  # noqa: E501

            Delete a label  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.delete_object_label(label_id, async_req=True)
            >>> result = thread.get()

            Args:
                label_id (str): A label Id

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['label_id'] = \
                label_id
            return self.call_with_http_info(**kwargs)

        if self.delete_object_label is None:
            self.delete_object_label = _Endpoint(
                settings={
                    'response_type': None,
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labels/{label_id}',
                    'operation_id': 'delete_object_label',
                    'http_method': 'DELETE',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'label_id',
                        'org_id',
                    ],
                    'required': [
                        'label_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'label_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'label_id': 'label_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'label_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__delete_object_label
            )

        def __get_labelled_object(
            self,
            labelled_object_id,
            **kwargs
        ):
            """Get a labelled object  # noqa: E501

            Get a labelled object  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_labelled_object(labelled_object_id, async_req=True)
            >>> result = thread.get()

            Args:
                labelled_object_id (str): A labelled object ID

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                LabelledObject
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['labelled_object_id'] = \
                labelled_object_id
            return self.call_with_http_info(**kwargs)

        if self.get_labelled_object is None:
            self.get_labelled_object = _Endpoint(
                settings={
                    'response_type': (LabelledObject,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects/{labelled_object_id}',
                    'operation_id': 'get_labelled_object',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'labelled_object_id',
                        'org_id',
                    ],
                    'required': [
                        'labelled_object_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'labelled_object_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'labelled_object_id': 'labelled_object_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'labelled_object_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_labelled_object
            )

        def __get_object_label(
            self,
            label_id,
            **kwargs
        ):
            """Get a label  # noqa: E501

            Get a label  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.get_object_label(label_id, async_req=True)
            >>> result = thread.get()

            Args:
                label_id (str): A label Id

            Keyword Args:
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Label
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['label_id'] = \
                label_id
            return self.call_with_http_info(**kwargs)

        if self.get_object_label is None:
            self.get_object_label = _Endpoint(
                settings={
                    'response_type': (Label,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labels/{label_id}',
                    'operation_id': 'get_object_label',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'label_id',
                        'org_id',
                    ],
                    'required': [
                        'label_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'label_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'label_id': 'label_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'label_id': 'path',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__get_object_label
            )

        def __list_labelled_objects(
            self,
            **kwargs
        ):
            """List all labelled objects  # noqa: E501

            List all labelled objects matching the provided query parameters. Perform keyset pagination by setting the page_at_id parameter to the id for the next page to fetch. Set it to `\"\"` to start from the beginning. Searching for label_id will return any tree that references that label.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_labelled_objects(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                includes_any_label ([LabelName]): A list of labels to match against. Matches objects with any of the labels. [optional]
                excludes_any_label ([LabelName]): A list of labels to match against. Matches objects with none of the labels. [optional]
                object_id (str): search by object id. [optional]
                object_type (ObjectType): An object type. [optional]
                object_types ([ObjectType]): A list of object types. Returns all items which match at least one of the types. . [optional]
                object_ids ([str]): A list of object IDs. Returns all items which match at least one of the . . [optional]
                org_ids ([str]): The list of org ids to search for. Each org will be searched for independently.. [optional]
                page_at_id (str): Pagination based query with the id as the key. To get the initial entries supply an empty string. On subsequent requests, supply the `page_at_id` field from the list response. . [optional]
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListLabelledObjectsResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_labelled_objects is None:
            self.list_labelled_objects = _Endpoint(
                settings={
                    'response_type': (ListLabelledObjectsResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects',
                    'operation_id': 'list_labelled_objects',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'includes_any_label',
                        'excludes_any_label',
                        'object_id',
                        'object_type',
                        'object_types',
                        'object_ids',
                        'org_ids',
                        'page_at_id',
                        'org_id',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'includes_any_label':
                            ([LabelName],),
                        'excludes_any_label':
                            ([LabelName],),
                        'object_id':
                            (str,),
                        'object_type':
                            (ObjectType,),
                        'object_types':
                            ([ObjectType],),
                        'object_ids':
                            ([str],),
                        'org_ids':
                            ([str],),
                        'page_at_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'includes_any_label': 'includes_any_label',
                        'excludes_any_label': 'excludes_any_label',
                        'object_id': 'object_id',
                        'object_type': 'object_type',
                        'object_types': 'object_types',
                        'object_ids': 'object_ids',
                        'org_ids': 'org_ids',
                        'page_at_id': 'page_at_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'limit': 'query',
                        'includes_any_label': 'query',
                        'excludes_any_label': 'query',
                        'object_id': 'query',
                        'object_type': 'query',
                        'object_types': 'query',
                        'object_ids': 'query',
                        'org_ids': 'query',
                        'page_at_id': 'query',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                        'includes_any_label': 'multi',
                        'excludes_any_label': 'multi',
                        'object_types': 'multi',
                        'object_ids': 'multi',
                        'org_ids': 'multi',
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_labelled_objects
            )

        def __list_object_labels(
            self,
            **kwargs
        ):
            """List all labels  # noqa: E501

            List all labels matching the provided query parameters. Perform keyset pagination by setting the page_at_id parameter to the id for the next page to fetch. Set it to `\"\"` to start from the beginning.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.list_object_labels(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): limit the number of rows in the response. [optional] if omitted the server will use the default value of 500
                label_name (LabelName): A label name. [optional]
                page_at_id (str): Pagination based query with the id as the key. To get the initial entries supply an empty string. On subsequent requests, supply the `page_at_id` field from the list response. . [optional]
                org_id (str): Organisation Unique identifier. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ListLabelsResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        if self.list_object_labels is None:
            self.list_object_labels = _Endpoint(
                settings={
                    'response_type': (ListLabelsResponse,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labels',
                    'operation_id': 'list_object_labels',
                    'http_method': 'GET',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'limit',
                        'label_name',
                        'page_at_id',
                        'org_id',
                    ],
                    'required': [],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                        'limit',
                    ]
                },
                root_map={
                    'validations': {
                        ('limit',): {

                            'inclusive_maximum': 500,
                            'inclusive_minimum': 1,
                        },
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'limit':
                            (int,),
                        'label_name':
                            (LabelName,),
                        'page_at_id':
                            (str,),
                        'org_id':
                            (str,),
                    },
                    'attribute_map': {
                        'limit': 'limit',
                        'label_name': 'label_name',
                        'page_at_id': 'page_at_id',
                        'org_id': 'org_id',
                    },
                    'location_map': {
                        'limit': 'query',
                        'label_name': 'query',
                        'page_at_id': 'query',
                        'org_id': 'query',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [],
                },
                api_client=api_client,
                callable=__list_object_labels
            )

        def __replace_labelled_object(
            self,
            labelled_object_id,
            **kwargs
        ):
            """update a labelled object  # noqa: E501

            update a labelled object  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_labelled_object(labelled_object_id, async_req=True)
            >>> result = thread.get()

            Args:
                labelled_object_id (str): A labelled object ID

            Keyword Args:
                labelled_object (LabelledObject): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                LabelledObject
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['labelled_object_id'] = \
                labelled_object_id
            return self.call_with_http_info(**kwargs)

        if self.replace_labelled_object is None:
            self.replace_labelled_object = _Endpoint(
                settings={
                    'response_type': (LabelledObject,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labelled_objects/{labelled_object_id}',
                    'operation_id': 'replace_labelled_object',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'labelled_object_id',
                        'labelled_object',
                    ],
                    'required': [
                        'labelled_object_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'labelled_object_id':
                            (str,),
                        'labelled_object':
                            (LabelledObject,),
                    },
                    'attribute_map': {
                        'labelled_object_id': 'labelled_object_id',
                    },
                    'location_map': {
                        'labelled_object_id': 'path',
                        'labelled_object': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_labelled_object
            )

        def __replace_object_label(
            self,
            label_id,
            **kwargs
        ):
            """update a label  # noqa: E501

            update a label  # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.replace_object_label(label_id, async_req=True)
            >>> result = thread.get()

            Args:
                label_id (str): A label Id

            Keyword Args:
                label (Label): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Label
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['label_id'] = \
                label_id
            return self.call_with_http_info(**kwargs)

        if self.replace_object_label is None:
            self.replace_object_label = _Endpoint(
                settings={
                    'response_type': (Label,),
                    'auth': [
                        'token-valid'
                    ],
                    'endpoint_path': '/v1/labels/{label_id}',
                    'operation_id': 'replace_object_label',
                    'http_method': 'PUT',
                    'servers': None,
                },
                params_map={
                    'all': [
                        'label_id',
                        'label',
                    ],
                    'required': [
                        'label_id',
                    ],
                    'nullable': [
                    ],
                    'enum': [
                    ],
                    'validation': [
                    ]
                },
                root_map={
                    'validations': {
                    },
                    'allowed_values': {
                    },
                    'openapi_types': {
                        'label_id':
                            (str,),
                        'label':
                            (Label,),
                    },
                    'attribute_map': {
                        'label_id': 'label_id',
                    },
                    'location_map': {
                        'label_id': 'path',
                        'label': 'body',
                    },
                    'collection_format_map': {
                    }
                },
                headers_map={
                    'accept': [
                        'application/json'
                    ],
                    'content_type': [
                        'application/json'
                    ]
                },
                api_client=api_client,
                callable=__replace_object_label
            )

    bulk_delete_labelled_objects = None 
    create_labelled_object = None 
    create_labelled_object_label = None 
    create_object_label = None 
    delete_labelled_object = None 
    delete_labelled_object_label = None 
    delete_object_label = None 
    get_labelled_object = None 
    get_object_label = None 
    list_labelled_objects = None 
    list_object_labels = None 
    replace_labelled_object = None 
    replace_object_label = None 



import json
import os
import certifi

from .agilicus_api.api_client import ApiClient
from .agilicus_api.configuration import Configuration


class _ApiClientWrapper(ApiClient):
    def __init__(self, configuration: Configuration = None, **kwargs):
        if not configuration:
            configuration = Configuration()

        cert_path = os.environ.get("SSL_CERT_FILE")

        if cert_path and not configuration.ssl_ca_cert:
            configuration.ssl_ca_cert = cert_path
        elif configuration.ssl_ca_cert is None:
            configuration.ssl_ca_cert = certifi.where()

        super().__init__(configuration=configuration, **kwargs)

    def deserialize_dict(self, data: dict, response_type):
        return self.deserialize_json(json.dumps(data), response_type)

    def deserialize_json(self, json_data: str, response_type):
        response = _Response(json_data)
        return self.deserialize(response, response_type, True)


def patched_api_client():
    return _ApiClientWrapper


class _Response:
    def __init__(self, data):
        self.data = data



from unittest.mock import MagicMock

class LaunchersApiMock:

    def __init__(self):
        self.mock_create_launcher = MagicMock()
        self.mock_delete_launcher = MagicMock()
        self.mock_get_launcher = MagicMock()
        self.mock_list_launchers = MagicMock()
        self.mock_replace_launcher = MagicMock()

    def create_launcher(self, *args, **kwargs):
        """
        This method mocks the original api LaunchersApi.create_launcher with MagicMock.
        """
        return self.mock_create_launcher(self, *args, **kwargs)

    def delete_launcher(self, *args, **kwargs):
        """
        This method mocks the original api LaunchersApi.delete_launcher with MagicMock.
        """
        return self.mock_delete_launcher(self, *args, **kwargs)

    def get_launcher(self, *args, **kwargs):
        """
        This method mocks the original api LaunchersApi.get_launcher with MagicMock.
        """
        return self.mock_get_launcher(self, *args, **kwargs)

    def list_launchers(self, *args, **kwargs):
        """
        This method mocks the original api LaunchersApi.list_launchers with MagicMock.
        """
        return self.mock_list_launchers(self, *args, **kwargs)

    def replace_launcher(self, *args, **kwargs):
        """
        This method mocks the original api LaunchersApi.replace_launcher with MagicMock.
        """
        return self.mock_replace_launcher(self, *args, **kwargs)




"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2023.10.24
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from agilicus_api.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
)
from ..model_utils import OpenApiModel
from agilicus_api.exceptions import ApiAttributeError


def lazy_import():
    from agilicus_api.model.feature_flag import FeatureFlag
    from agilicus_api.model.k8s_slug import K8sSlug
    from agilicus_api.model.organisation_capabilities import OrganisationCapabilities
    from agilicus_api.model.organisation_owner_config import OrganisationOwnerConfig
    from agilicus_api.model.organisation_state_selector import OrganisationStateSelector
    from agilicus_api.model.organisation_status import OrganisationStatus
    globals()['FeatureFlag'] = FeatureFlag
    globals()['K8sSlug'] = K8sSlug
    globals()['OrganisationCapabilities'] = OrganisationCapabilities
    globals()['OrganisationOwnerConfig'] = OrganisationOwnerConfig
    globals()['OrganisationStateSelector'] = OrganisationStateSelector
    globals()['OrganisationStatus'] = OrganisationStatus


class Organisation(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
        ('external_id',): {
            'max_length': 100,
        },
        ('organisation',): {
            'max_length': 100,
        },
        ('issuer',): {
            'max_length': 100,
        },
        ('subdomain',): {
            'max_length': 100,
        },
        ('contact_email',): {
            'max_length': 100,
        },
        ('trust_on_first_use_duration',): {
            'inclusive_minimum': 0,
        },
        ('shard',): {
            'max_length': 50,
        },
        ('cluster',): {
            'max_length': 32,
        },
        ('product_label_override',): {
            'max_length': 32,
            'regex': {
                'pattern': r'^[a-zA-Z0-9-_.:]+$',  # noqa: E501
            },
        },
    }

    @property
    def id(self):
       return self.get("id")

    @id.setter
    def id(self, new_value):
       self.id = new_value

    @property
    def all_users_group_id(self):
       return self.get("all_users_group_id")

    @all_users_group_id.setter
    def all_users_group_id(self, new_value):
       self.all_users_group_id = new_value

    @property
    def all_users_all_suborgs_group_id(self):
       return self.get("all_users_all_suborgs_group_id")

    @all_users_all_suborgs_group_id.setter
    def all_users_all_suborgs_group_id(self, new_value):
       self.all_users_all_suborgs_group_id = new_value

    @property
    def all_users_direct_suborgs_group_id(self):
       return self.get("all_users_direct_suborgs_group_id")

    @all_users_direct_suborgs_group_id.setter
    def all_users_direct_suborgs_group_id(self, new_value):
       self.all_users_direct_suborgs_group_id = new_value

    @property
    def auto_created_users_group_id(self):
       return self.get("auto_created_users_group_id")

    @auto_created_users_group_id.setter
    def auto_created_users_group_id(self, new_value):
       self.auto_created_users_group_id = new_value

    @property
    def external_id(self):
       return self.get("external_id")

    @external_id.setter
    def external_id(self, new_value):
       self.external_id = new_value

    @property
    def organisation(self):
       return self.get("organisation")

    @organisation.setter
    def organisation(self, new_value):
       self.organisation = new_value

    @property
    def issuer(self):
       return self.get("issuer")

    @issuer.setter
    def issuer(self, new_value):
       self.issuer = new_value

    @property
    def issuer_id(self):
       return self.get("issuer_id")

    @issuer_id.setter
    def issuer_id(self, new_value):
       self.issuer_id = new_value

    @property
    def subdomain(self):
       return self.get("subdomain")

    @subdomain.setter
    def subdomain(self, new_value):
       self.subdomain = new_value

    @property
    def name_slug(self):
       return self.get("name_slug")

    @name_slug.setter
    def name_slug(self, new_value):
       self.name_slug = new_value

    @property
    def created(self):
       return self.get("created")

    @created.setter
    def created(self, new_value):
       self.created = new_value

    @property
    def updated(self):
       return self.get("updated")

    @updated.setter
    def updated(self, new_value):
       self.updated = new_value

    @property
    def contact_id(self):
       return self.get("contact_id")

    @contact_id.setter
    def contact_id(self, new_value):
       self.contact_id = new_value

    @property
    def contact_email(self):
       return self.get("contact_email")

    @contact_email.setter
    def contact_email(self, new_value):
       self.contact_email = new_value

    @property
    def parent_id(self):
       return self.get("parent_id")

    @parent_id.setter
    def parent_id(self, new_value):
       self.parent_id = new_value

    @property
    def root_org_id(self):
       return self.get("root_org_id")

    @root_org_id.setter
    def root_org_id(self, new_value):
       self.root_org_id = new_value

    @property
    def auto_create(self):
       return self.get("auto_create")

    @auto_create.setter
    def auto_create(self, new_value):
       self.auto_create = new_value

    @property
    def trust_on_first_use_duration(self):
       return self.get("trust_on_first_use_duration")

    @trust_on_first_use_duration.setter
    def trust_on_first_use_duration(self, new_value):
       self.trust_on_first_use_duration = new_value

    @property
    def feature_flags(self):
       return self.get("feature_flags")

    @feature_flags.setter
    def feature_flags(self, new_value):
       self.feature_flags = new_value

    @property
    def admin_state(self):
       return self.get("admin_state")

    @admin_state.setter
    def admin_state(self, new_value):
       self.admin_state = new_value

    @property
    def status(self):
       return self.get("status")

    @status.setter
    def status(self, new_value):
       self.status = new_value

    @property
    def billing_account_id(self):
       return self.get("billing_account_id")

    @billing_account_id.setter
    def billing_account_id(self, new_value):
       self.billing_account_id = new_value

    @property
    def billing_subscription_id(self):
       return self.get("billing_subscription_id")

    @billing_subscription_id.setter
    def billing_subscription_id(self, new_value):
       self.billing_subscription_id = new_value

    @property
    def shard(self):
       return self.get("shard")

    @shard.setter
    def shard(self, new_value):
       self.shard = new_value

    @property
    def cluster(self):
       return self.get("cluster")

    @cluster.setter
    def cluster(self, new_value):
       self.cluster = new_value

    @property
    def configured_capabilities(self):
       return self.get("configured_capabilities")

    @configured_capabilities.setter
    def configured_capabilities(self, new_value):
       self.configured_capabilities = new_value

    @property
    def owner_config(self):
       return self.get("owner_config")

    @owner_config.setter
    def owner_config(self, new_value):
       self.owner_config = new_value

    @property
    def product_label_override(self):
       return self.get("product_label_override")

    @product_label_override.setter
    def product_label_override(self, new_value):
       self.product_label_override = new_value

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'id': (str,),  # noqa: E501
            'all_users_group_id': (str,),  # noqa: E501
            'all_users_all_suborgs_group_id': (str,),  # noqa: E501
            'all_users_direct_suborgs_group_id': (str,),  # noqa: E501
            'auto_created_users_group_id': (str,),  # noqa: E501
            'external_id': (str,),  # noqa: E501
            'organisation': (str,),  # noqa: E501
            'issuer': (str,),  # noqa: E501
            'issuer_id': (str,),  # noqa: E501
            'subdomain': (str,),  # noqa: E501
            'name_slug': (K8sSlug,),  # noqa: E501
            'created': (datetime,),  # noqa: E501
            'updated': (datetime,),  # noqa: E501
            'contact_id': (str,),  # noqa: E501
            'contact_email': (str,),  # noqa: E501
            'parent_id': (str,),  # noqa: E501
            'root_org_id': (str,),  # noqa: E501
            'auto_create': (bool,),  # noqa: E501
            'trust_on_first_use_duration': (int,),  # noqa: E501
            'feature_flags': ([FeatureFlag],),  # noqa: E501
            'admin_state': (OrganisationStateSelector,),  # noqa: E501
            'status': (OrganisationStatus,),  # noqa: E501
            'billing_account_id': (str,),  # noqa: E501
            'billing_subscription_id': (str,),  # noqa: E501
            'shard': (str,),  # noqa: E501
            'cluster': (str,),  # noqa: E501
            'configured_capabilities': (OrganisationCapabilities,),  # noqa: E501
            'owner_config': (OrganisationOwnerConfig,),  # noqa: E501
            'product_label_override': (str,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None



    attribute_map = {
        'id': 'id',  # noqa: E501
        'all_users_group_id': 'all_users_group_id',  # noqa: E501
        'all_users_all_suborgs_group_id': 'all_users_all_suborgs_group_id',  # noqa: E501
        'all_users_direct_suborgs_group_id': 'all_users_direct_suborgs_group_id',  # noqa: E501
        'auto_created_users_group_id': 'auto_created_users_group_id',  # noqa: E501
        'external_id': 'external_id',  # noqa: E501
        'organisation': 'organisation',  # noqa: E501
        'issuer': 'issuer',  # noqa: E501
        'issuer_id': 'issuer_id',  # noqa: E501
        'subdomain': 'subdomain',  # noqa: E501
        'name_slug': 'name_slug',  # noqa: E501
        'created': 'created',  # noqa: E501
        'updated': 'updated',  # noqa: E501
        'contact_id': 'contact_id',  # noqa: E501
        'contact_email': 'contact_email',  # noqa: E501
        'parent_id': 'parent_id',  # noqa: E501
        'root_org_id': 'root_org_id',  # noqa: E501
        'auto_create': 'auto_create',  # noqa: E501
        'trust_on_first_use_duration': 'trust_on_first_use_duration',  # noqa: E501
        'feature_flags': 'feature_flags',  # noqa: E501
        'admin_state': 'admin_state',  # noqa: E501
        'status': 'status',  # noqa: E501
        'billing_account_id': 'billing_account_id',  # noqa: E501
        'billing_subscription_id': 'billing_subscription_id',  # noqa: E501
        'shard': 'shard',  # noqa: E501
        'cluster': 'cluster',  # noqa: E501
        'configured_capabilities': 'configured_capabilities',  # noqa: E501
        'owner_config': 'owner_config',  # noqa: E501
        'product_label_override': 'product_label_override',  # noqa: E501
    }

    read_only_vars = {
        'id',  # noqa: E501
        'created',  # noqa: E501
        'updated',  # noqa: E501
        'contact_email',  # noqa: E501
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """Organisation - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): Unique identifier. [optional]  # noqa: E501
            all_users_group_id (str): group id of group containing this organisations all users. [optional]  # noqa: E501
            all_users_all_suborgs_group_id (str): group id of group containing this organisations all users including all sub organisations. [optional]  # noqa: E501
            all_users_direct_suborgs_group_id (str): group id of group containing this organisations all users including only direct sub organisations. [optional]  # noqa: E501
            auto_created_users_group_id (str): group id of group containing users automatically created when they logged in. [optional]  # noqa: E501
            external_id (str): External unique identifier. [optional]  # noqa: E501
            organisation (str): organisation name. [optional]  # noqa: E501
            issuer (str): connect id issuer. [optional]  # noqa: E501
            issuer_id (str): Unique identifier. [optional]  # noqa: E501
            subdomain (str): Organisations subdomain. Note this must be unique accross all organisations. [optional]  # noqa: E501
            name_slug (K8sSlug): [optional]  # noqa: E501
            created (datetime): Creation time. [optional]  # noqa: E501
            updated (datetime): Update time. [optional]  # noqa: E501
            contact_id (str): GUID of the organisation admin. [optional]  # noqa: E501
            contact_email (str): The email address of the contact.. [optional]  # noqa: E501
            parent_id (str): parent organisation id. [optional]  # noqa: E501
            root_org_id (str): The id of the organisation at the root of this organisation hierarchy. For example, if A is the parent of B, and B is the parent of C, then A would be the root organisation of A, B and C. Note that this field will be ignored if changed. . [optional]  # noqa: E501
            auto_create (bool): Auto-creates a user. [optional] if omitted the server will use the default value of False  # noqa: E501
            trust_on_first_use_duration (int): The time window in seconds which represents the period of time for which a new user is eligible for trust on first use enrollment. The duration will be applied from the user's created date, or if the user's enrollment period was reset by an administrator. . [optional] if omitted the server will use the default value of 86400  # noqa: E501
            feature_flags ([FeatureFlag]): A list of features to enable or disable. This is mostly for advanced use. No guarantees exist that a given feature will always exist by a given name, or that it will be configurable. . [optional]  # noqa: E501
            admin_state (OrganisationStateSelector): [optional]  # noqa: E501
            status (OrganisationStatus): [optional]  # noqa: E501
            billing_account_id (str): The billing account id. [optional]  # noqa: E501
            billing_subscription_id (str): The billing subscription id. [optional]  # noqa: E501
            shard (str): The shard to use on the agilicus platform. This provides logical seperation on the agilicus platform. This can be used to prevent access to applications between suborgs in the same organisation. . [optional]  # noqa: E501
            cluster (str): The agilicus cluster/region to host the platform on. This provides information to determine what physical location any services provided should run from. Consult the list of Regions for the possible values to set here. Changing this value will cause resources to move to the new cluster. Changing the setting will cause a few minutes of downtime while resources are moved. You will have to change the cname for your domain in order to attract traffic to the correct location. . [optional]  # noqa: E501
            configured_capabilities (OrganisationCapabilities): [optional]  # noqa: E501
            owner_config (OrganisationOwnerConfig): [optional]  # noqa: E501
            product_label_override (str): Override the default product label. . [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    def __python_set(val):
        return set(val)
 
    required_properties = __python_set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """Organisation - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            id (str): Unique identifier. [optional]  # noqa: E501
            all_users_group_id (str): group id of group containing this organisations all users. [optional]  # noqa: E501
            all_users_all_suborgs_group_id (str): group id of group containing this organisations all users including all sub organisations. [optional]  # noqa: E501
            all_users_direct_suborgs_group_id (str): group id of group containing this organisations all users including only direct sub organisations. [optional]  # noqa: E501
            auto_created_users_group_id (str): group id of group containing users automatically created when they logged in. [optional]  # noqa: E501
            external_id (str): External unique identifier. [optional]  # noqa: E501
            organisation (str): organisation name. [optional]  # noqa: E501
            issuer (str): connect id issuer. [optional]  # noqa: E501
            issuer_id (str): Unique identifier. [optional]  # noqa: E501
            subdomain (str): Organisations subdomain. Note this must be unique accross all organisations. [optional]  # noqa: E501
            name_slug (K8sSlug): [optional]  # noqa: E501
            created (datetime): Creation time. [optional]  # noqa: E501
            updated (datetime): Update time. [optional]  # noqa: E501
            contact_id (str): GUID of the organisation admin. [optional]  # noqa: E501
            contact_email (str): The email address of the contact.. [optional]  # noqa: E501
            parent_id (str): parent organisation id. [optional]  # noqa: E501
            root_org_id (str): The id of the organisation at the root of this organisation hierarchy. For example, if A is the parent of B, and B is the parent of C, then A would be the root organisation of A, B and C. Note that this field will be ignored if changed. . [optional]  # noqa: E501
            auto_create (bool): Auto-creates a user. [optional] if omitted the server will use the default value of False  # noqa: E501
            trust_on_first_use_duration (int): The time window in seconds which represents the period of time for which a new user is eligible for trust on first use enrollment. The duration will be applied from the user's created date, or if the user's enrollment period was reset by an administrator. . [optional] if omitted the server will use the default value of 86400  # noqa: E501
            feature_flags ([FeatureFlag]): A list of features to enable or disable. This is mostly for advanced use. No guarantees exist that a given feature will always exist by a given name, or that it will be configurable. . [optional]  # noqa: E501
            admin_state (OrganisationStateSelector): [optional]  # noqa: E501
            status (OrganisationStatus): [optional]  # noqa: E501
            billing_account_id (str): The billing account id. [optional]  # noqa: E501
            billing_subscription_id (str): The billing subscription id. [optional]  # noqa: E501
            shard (str): The shard to use on the agilicus platform. This provides logical seperation on the agilicus platform. This can be used to prevent access to applications between suborgs in the same organisation. . [optional]  # noqa: E501
            cluster (str): The agilicus cluster/region to host the platform on. This provides information to determine what physical location any services provided should run from. Consult the list of Regions for the possible values to set here. Changing this value will cause resources to move to the new cluster. Changing the setting will cause a few minutes of downtime while resources are moved. You will have to change the cname for your domain in order to attract traffic to the correct location. . [optional]  # noqa: E501
            configured_capabilities (OrganisationCapabilities): [optional]  # noqa: E501
            owner_config (OrganisationOwnerConfig): [optional]  # noqa: E501
            product_label_override (str): Override the default product label. . [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            raise ApiTypeError(
                "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                    args,
                    self.__class__.__name__,
                ),
                path_to_item=_path_to_item,
                valid_classes=(self.__class__,),
            )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")

